/*
 * Sample bot in C.
 * gcc randombot_c.c -std=c99
 */

#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>

/*
 * Implement choose_new_target and choose_train
 *
 * You have the following variables at your disposal:
 * num_players: number of bots
 * num_skills: number of skills
 * target: the current target
 * skill_difficulty: the difficulties of the skills
 * days_remaining: list for each player of this players skills. For each player there is a list of missing training per skill
 *    (i.e. it is the other way than in the description. It starts at s_i and ends at 0 once mastered)
 * player_selection: the last action for each player (i.e. which skill was chosen)
 * total_score: scores for each player total over all games
 * score: scores for each player of the current game
 * skill_unmastered: which skills have not yet a winner
 */

#define MAX_N 100
#define MAX_M 1000
#define PICK_TARGET -1
#define RESET -2
#define GAMEOVER -3

int num_players, num_skills, target;
int skill_difficulty[MAX_M], player_selection[MAX_M], winners[MAX_N];
double total_score[MAX_N], score[MAX_N];
bool skill_unmastered[MAX_M];
int days_remaining[MAX_N][MAX_M];

/* Pick a new target randomly. */
int choose_new_target() {
  int selection = -1, sample_size = 0;
  for (int i = 0; i < num_skills; i++) {
    /* Is this target available? */
    if (skill_unmastered[i]) {
      /* Reservoir sampling. */
      ++sample_size;
      if (rand() % sample_size == 0) {
        selection = i;
      }
    }
  }
  return selection;
}

/* Pick a random skill to train in this round. */
int choose_train() {
  int selection = -1, sample_size = 0;
  for (int i = 0; i < num_skills; i++) {
    /* Is this target feasable? */
    if (days_remaining[0][i] > 1 || (days_remaining[0][i] == 1 && target == i)) {
      /* Reservoir sampling. */
      ++sample_size;
      if (rand() % sample_size == 0) {
        selection = i;
      }
    }
  }
  return selection;
}

int main() {
  /* Arbitrary random seed. Fixed for reproducibility. */
  srand(42);

  for (int game = 0; ; game++) {
    /* Read game config. */
    scanf("%d%d", &num_players, &num_skills);
    for (int i = 0; i < num_skills; i++) {
      scanf("%d", &skill_difficulty[i]);
      skill_unmastered[i] = true;
    }
    for (int i = 0; i < num_players; i++) {
      score[i] = 0.0;
      /* Only initialize the total scores on the first game. */
      if (game == 0) {
        total_score[i] = 0.0;
      }
      for (int j = 0; j < num_skills; j++) {
        days_remaining[i][j] = skill_difficulty[j];
      }
    }

    /* Play the game. */
    for (;;) {
      int target_or_status; scanf("%d", &target_or_status);

      /* Did the game finish? */
      if (target_or_status == GAMEOVER) {
        return EXIT_SUCCESS;
      }

      /* Update scores. */
      int num_winners = 0;
      for (int i = 0; i < num_players; i++) {
        scanf("%d", &player_selection[i]);
        if (player_selection[i] >= 0) {
          days_remaining[i][player_selection[i]] -= 1;
          /* Do we have a winner? */
          if (days_remaining[i][player_selection[i]] == 0) {
            winners[num_winners] = 0;
            ++num_winners;
          }
        }
      }
      if (num_winners > 0) {
        /* Skill was reached. */
        skill_unmastered[target] = false;

        /* Update scores of winners. */
        double reward = 1.0/num_winners;
        for (int i = 0; i < num_winners; i++) {
          score[winners[i]] += reward;
          total_score[winners[i]] += reward;
        }
      }

      /* Did we finish this round? */
      if (target_or_status == RESET) {
        break;

      /* Can we pick the next target? */
      } else if (target_or_status == PICK_TARGET) {
        target = choose_new_target();
        printf("%d\n", target);

      } else {
        target = target_or_status;
      }

      printf("%d\n", choose_train());
      fflush(stdout);
    }
  }

  return EXIT_SUCCESS;
}
