extern crate rand;

use std::io::BufRead;
use rand::Rng;

/// You have the following variables at your disposal:
#[allow(dead_code)]
struct GameState<'a> {
    /// number of players
    num_players: usize,

    /// number of skills
    num_skills: usize,

    /// the current target, or -1 if there is currently none
    target: Option<usize>,

    /// the difficulties of the skills
    skill_difficulty: Vec<u32>,

    /// the last action for each player (i.e. which skill was chosen)
    player_selection: Vec<usize>,

    /// scores for each player total over all games
    total_score: &'a mut Vec<f64>,

    /// scores for each player of the current game
    score: Vec<f64>,

    /// which skills have not yet a winner
    skill_unmastered: Vec<bool>,

    /// list for each player of this players skill_difficulty. For each player there is a list of missing training per skill_difficulty
    /// (i.e. it is the other way than in the description. It starts at s_i and ends at 0 once mastered)
    days_remaining: Vec<Vec<u32>>
}

/// Implement this trait
trait Bot {
    /// choose a skill to train
    fn choose_train(&mut self, gs: &GameState) -> usize;

    /// select a new target
    fn choose_new_target(&mut self, gs: &GameState) -> usize;
}

struct RandomBot; // a smart bot would store additional information here

impl RandomBot {
    fn new() -> RandomBot { RandomBot {} }
}

impl Bot for RandomBot {
    fn choose_train(&mut self, gs: &GameState) -> usize {
        *rand::thread_rng().choose(
            &gs.days_remaining[0].iter()
                .enumerate()
                .filter(|&(skill, &remaining)| remaining > 1 || (remaining == 1 && skill == gs.target.unwrap()))
                .map(|(skill, _)| skill)
                .collect::<Vec<_>>()
        ).unwrap()
    }

    fn choose_new_target(&mut self, gs: &GameState) -> usize {
        *rand::thread_rng().choose(
            &gs.skill_unmastered.iter()
                .enumerate()
                .filter(|&(_, &ok)| ok)
                .map(|(skill, _)| skill)
                .collect::<Vec<_>>()
        ).unwrap()
    }
}

fn main() {
    let stdin = std::io::stdin();
    let mut lines = stdin.lock().lines().map(|l| l.unwrap());

    let mut total_score = Vec::new();
    let mut bot = RandomBot::new();

    loop {
        let (n, m) = {
            let line = lines.next().unwrap();
            let mut vals = line.split_whitespace().map(|n| n.parse().unwrap());
            (vals.next().unwrap(), vals.next().unwrap())
        };
        if total_score.len() == 0 {
            total_score = std::iter::repeat(0.).take(n).collect();
        }
        let skills : Vec<u32> = lines.next().unwrap().split_whitespace().map(|n| n.parse().unwrap()).collect();
        let mut gs = GameState {
            num_players: n,
            num_skills: m,
            target: None,
            skill_unmastered: std::iter::repeat(true).take(m).collect(),
            score: std::iter::repeat(0.).take(m).collect(),
            player_selection: Vec::new(),
            total_score: &mut total_score,
            days_remaining: std::iter::repeat(skills.clone()).take(n).collect(),
            skill_difficulty: skills,
        };

        loop {
            let target : i32 = lines.next().unwrap().parse().unwrap();
            if target == -3 { return } // game over

            match gs.target {
                None => { lines.next(); } // all previous player_selection will be -1, no need to parse
                Some(t) => {
                    gs.player_selection = lines.next().unwrap().split_whitespace().map(|n| n.parse().unwrap()).collect();
                    for (player, &action) in gs.player_selection.iter().enumerate() {
                        gs.days_remaining[player][action] -= 1;
                    }
                    let winners = gs.days_remaining.iter().enumerate()
                        .filter(|&(i, s)| s[gs.player_selection[i]] == 0)
                        .map(|(i,_)| i)
                        .collect::<Vec<_>>();
                    if winners.len() > 0 {
                        gs.skill_unmastered[t] = false;
                        let points = 1.0/(winners.len() as f64);
                        for w in winners {
                            gs.score[w] += points;
                            gs.total_score[w] += points;
                        }
                    }
                }
            }

            let target = match target {
                -1 => {
                    let t = bot.choose_new_target(&gs);
                    println!("{}", t);
                    t
                },
                -2 => break, // start new round
                t if t >= 0 => t as usize,
                _ => unreachable!()
            };
            gs.target = Some(target);
            println!("{}", bot.choose_train(&gs));
        }
    }
}
